#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "config.h"
#include "types.h"
#include "lib.h"
#include "string.h"

static struct string strings[STRING_MAXNUM];

static string_t head = NULL;
static string_t pool = NULL;

int string_check(void)
{
  string_t string;
  int n = 0;

  if (head)
    return NLL_ERRCODE_STRING_NOT_EMPTY;

  for (string = pool; string && (n < STRING_MAXNUM); string = string->next)
    n++;

  if (string || (n != STRING_MAXNUM))
    return NLL_ERRCODE_STRING_NOT_EMPTY;

  return 0;
}

int string_alloc(string_t *stringp)
{
  string_t string;

  if (!pool)
    return NLL_ERRCODE_STRING_BUFFER_OVER;

  string = pool;
  pool = pool->next;

  memset(string, 0, sizeof(*string));

  string->next = head;
  head = string;

  *stringp = string;

  return 0;
}

int string_free(string_t string)
{
  string_t next;

  for (; string; string = next) {
    next = string->next;

    string->next = pool;
    pool = string;
  }

  return 0;
}

int string_del(string_t string)
{
  string_t *p, next;

  for (p = &head; *p;) {
    if (*p == string) {
      next = (*p)->next;
      (*p)->next = NULL;
      string_free(*p);
      *p = next;
      continue;
    }
    p = &((*p)->next);
  }

  return 0;
}

int string_init(void)
{
  string_t string;
  int i;

  head = NULL;
  pool = NULL;
  memset(strings, 0, sizeof(strings));

  for (i = 0; i < STRING_MAXNUM; i++) {
    string = &strings[i];
    string->next = pool;
    pool = string;
  }

  return 0;
}

int string_print(string_t string)
{
  char *p, escape, c;

  printf("\"");

  for (p = string->s; *p; p++) {
    escape = 0;
    switch (*p) {
    case '\t': escape = 1; c = 't'; break;
    case '\n': escape = 1; c = 'n'; break;
    case '\v': escape = 1; c = 'v'; break;
    case '\r': escape = 1; c = 'r'; break;
    default: c = *p; break;
    }
    printf("%s%c", (escape ? "\\" : ""), c);
  }

  printf("\"");

  return 0;
}

int string_dump(string_t string)
{
  if (!string)
    string = head;

  printf("\n-- String dump --\n");

  for (; string; string = string->next) {
    string_print(string);
    printf("\n");
  }

  fflush(stdout);

  return 0;
}
