#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "term.h"

static struct termios save;
static int savefl;

static int savenum = 0;

int nll_term_start(void)
{
  struct termios sets;

  tcgetattr(STDIN_FILENO, &sets);

  save = sets;
  sets.c_lflag &= ~(ECHO | ICANON);
  sets.c_cc[VTIME] = 0;
  sets.c_cc[VMIN]  = 1;
  tcsetattr(STDIN_FILENO, TCSANOW, &sets);

  savefl = fcntl(STDIN_FILENO, F_GETFL, 0);
  fcntl(STDIN_FILENO, F_SETFL, savefl | O_NONBLOCK);

  savenum = 0;

  return 0;
}

int nll_term_restore(void)
{
  fcntl(STDIN_FILENO, F_SETFL, savefl);
  tcsetattr(STDIN_FILENO, TCSANOW, &save);
  return 0;
}

int nll_term_key(void)
{
  int r, i, escape = 0, k;
#define KEY_SAVENUM 8
  static int key[KEY_SAVENUM];
  unsigned char c;

  for (i = 0; i < 64; i++) {
    r = read(STDIN_FILENO, &c, 1);
    if (r <= 0)
      break;
    k = c;

    if (escape) {
      switch (escape) {
      case 0x1b:
	escape = 0;
	/* fall through */
      case 0x4f:
      case 0x5b:
      case 0x5b35:
      case 0x5b36:
	escape = (escape << 8) | k;
	switch (escape) {
	case NLL_TERM_KEY_END:
	case NLL_TERM_KEY_UP:
	case NLL_TERM_KEY_DOWN:
	case NLL_TERM_KEY_RIGHT:
	case NLL_TERM_KEY_LEFT:
	case NLL_TERM_KEY_PAGEUP:
	case NLL_TERM_KEY_PAGEDOWN:
	  k = escape;
	  escape = 0;
	  break;
	default:
	  continue;
	}
	break;

      default:
	escape = 0;
	continue;
      }
    } else if (k == 0x1b) {
      escape = k;
      continue;
    }

    if ((!savenum || (k != key[savenum - 1])) && (savenum < KEY_SAVENUM))
      key[savenum++] = k;
  }

  if (!savenum)
    return 0;

  k = key[0];
  for (i = 0; i < savenum - 1; i++)
    key[i] = key[i + 1];
  savenum--;

  return k;
}
