#ifndef __NLL_VALUE_H_INCLUDED__
#define __NLL_VALUE_H_INCLUDED__

typedef enum {
  VALUE_TYPE_UNDEFINED = 0,
  VALUE_TYPE_VALUE,
  VALUE_TYPE_ARRAY,
  VALUE_TYPE_INTEGER,
  VALUE_TYPE_STRING,
#ifdef NLL_FLOATING_POINT
  VALUE_TYPE_FLOAT,
#endif
} value_type_t;

struct value {
  struct value *next;
  value_type_t type;
  union {
    value_t value;
    struct {
      int number;
      array_t array;
    } array;
    int integer;
    string_t string;
#ifdef NLL_FLOATING_POINT
    double floating;
#endif
  } val;
};

int value_check(void);
int value_alloc(value_t *valuep);
int value_free(value_t value);
int value_init(void);
int value_clear(value_t value);
int value_copy_values(value_t dvalue, value_t svalue);
value_type_t value_get_type(value_t value);
int value_is_same(value_t value0, value_t value1);
int value_get_value(value_t value, value_t *valuep);
int value_set_value(value_t value, value_t _value);
int value_clear_value(value_t value);
int value_set_values(value_t value, value_t _value);
int value_get_array(value_t value, int *numberp, array_t *arrayp);
int value_set_array(value_t value, int number, array_t array);
int value_get_integer(value_t value, int *integerp);
int value_set_integer(value_t value, int integer);
int value_get_string(value_t value, char **stringp);
int value_set_string(value_t value, char *string);
#ifdef NLL_FLOATING_POINT
int value_get_float(value_t value, double *floatingp);
int value_set_float(value_t value, double floating);
#endif
int value_print(value_t value);
int value_dump(value_t value);

#endif
